#!/usr/bin/env python
# coding=utf-8
'''
Author: Liu Kun && 16031215@qq.com
Date: 2024-09-17 16:09:20
LastEditors: Liu Kun && 16031215@qq.com
LastEditTime: 2024-10-06 19:43:24
FilePath: \\Python\\My_Funcs\\OAFuncs\\OAFuncs\\__init__.py
Description:  
EditPlatform: vscode
ComputerInfo: XPS 15 9510
SystemInfo: Windows 11
Python Version: 3.11
'''

# 会导致OAFuncs直接导入所有函数，不符合模块化设计
from .oa_cmap import *
from .oa_data import *
from .oa_draw import *
from .oa_file import *
from .oa_nc import *
